#include "lcd.h"

void delay_ms(uint16_t time)
{
 while(time--) _delay_ms(1);
}

void setport(volatile uint8_t *outputregister, uint8_t output)
{
 *outputregister = output;
}

void setbit(volatile uint8_t *outputregister, uint8_t outputbit)
{
 *outputregister |= (1<<outputbit);
}

void unsetbit(volatile uint8_t *outputregister, uint8_t outputbit)
{
 *outputregister &= ~(1<<outputbit);
}

void LCD_SwapData(LCD *lcd)
{
 for(short i=0;i<4;i++)
 {
  lcd->DB[4+i].Data=lcd->DB[i].Data;
 }
}

void LCD_ToggleEnable(LCD *lcd)
{
 setbit(lcd->E.Port,lcd->E.Nummer);
 _delay_us(1);
 unsetbit(lcd->E.Port,lcd->E.Nummer);
}

void LCD_Set(LCD *lcd)
{
 for(short i=4;i<8;i++)
 {
  if(lcd->DB[i].Data==1) setbit(lcd->DB[i].Port,lcd->DB[i].Nummer);
  else unsetbit(lcd->DB[i].Port,lcd->DB[i].Nummer);
 }
}

void LCD_SetData(LCD *lcd,unsigned char data)
{
 unsigned char mask=0x01;
 
 for(int i=0;i<8;i++)
 {
  if(data & mask)
  {
   lcd->DB[i].Data=1;
  }
  else
  {
   lcd->DB[i].Data=0;
  }
  mask <<= 1;
 }

 LCD_Set(lcd);
}

void LCD_SendData(LCD *lcd)
{
 setbit(lcd->RS.Port,lcd->RS.Nummer);
 
 LCD_Set(lcd);
 
 LCD_ToggleEnable(lcd);
 
 LCD_SwapData(lcd);
 
 LCD_Set(lcd);
 
 LCD_ToggleEnable(lcd);
 
// unsetbit(lcd.RS.Port,lcd.RS.Nummer);
 _delay_us(25);
 _delay_us(25);
}

void LCD_SendCommand(LCD *lcd)
{
 unsetbit(lcd->RS.Port,lcd->RS.Nummer);
 
 LCD_Set(lcd);
 
 LCD_ToggleEnable(lcd);
 
 LCD_SwapData(lcd);
 
 LCD_Set(lcd);
 LCD_ToggleEnable(lcd);
 _delay_us(25);
 _delay_us(25);
}

void LCD_Clear(LCD *lcd)
{
 LCD_SetData(lcd,0b00000001);
 LCD_SendCommand(lcd);
 delay_ms(5);
}

void LCD_Init(LCD *lcd)
{
 
 delay_ms(50);					// Wait more than 15/40ms
 
 unsetbit(lcd->RS.Port,lcd->RS.Nummer);
 LCD_SetData(lcd,0b00110000);
 LCD_ToggleEnable(lcd);
 
 delay_ms(6);					// Wait more than 4.1ms
 
 LCD_ToggleEnable(lcd);
 
 delay_ms(1);					// Wait more than 100ms
 
 LCD_ToggleEnable(lcd);
 
 delay_ms(1);
 
 LCD_SetData(lcd,0b00100000);	// Function Set(4-Bit Mode)
 LCD_ToggleEnable(lcd);
 
 delay_ms(2);					// Wait ...
 
 LCD_SetData(lcd,0b00101000);	// Function Set(Lines, Font, 4-Bit Mode)
 LCD_SendCommand(lcd);
 
 LCD_SetData(lcd,0b00010100); 	//Cursor Move,Right Shift
 LCD_SendCommand(lcd);

 LCD_SetData(lcd,0b00001100);	// Display On
 LCD_SendCommand(lcd);
 
 LCD_SetData(lcd,0b00000110);	// Entry Mode Set
 LCD_SendCommand(lcd);

 LCD_Clear(lcd);
}
